/**
 * GLPI CVE Plugin - Main JavaScript
 * This file contains jQuery code for the CVE plugin functionality
 */

$(document).ready(function() {
  // Navigation handling
  $('.cve-nav-item').on('click', function(e) {
    e.preventDefault();
    const target = $(this).data('target');
    
    // Update active state
    $('.cve-nav-item').removeClass('active');
    $(this).addClass('active');
    
    // Show relevant content
    $('.cve-content').hide();
    $(`#${target}`).show();
    
    // Update URL hash for bookmarking
    window.location.hash = target;
  });
  
  // Initialize based on hash or default to dashboard
  const initPage = () => {
    let target = window.location.hash.substring(1) || 'cve-dashboard';
    $(`.cve-nav-item[data-target="${target}"]`).trigger('click');
  };
  
  initPage();
  
  // CVE Search functionality
  $('#cve-search').on('keyup', function() {
    const searchTerm = $(this).val().toLowerCase();
    filterCVEs(searchTerm, $('#severity-filter').val(), $('#status-filter').val());
  });
  
  // Severity and Status filters
  $('#severity-filter, #status-filter').on('change', function() {
    filterCVEs(
      $('#cve-search').val().toLowerCase(),
      $('#severity-filter').val(),
      $('#status-filter').val()
    );
  });
  
  // Table sorting
  $('.cve-sortable').on('click', function() {
    const column = $(this).data('column');
    const currentDir = $(this).hasClass('sort-asc') ? 'asc' : 'desc';
    const newDir = currentDir === 'asc' ? 'desc' : 'asc';
    
    // Update visual indicators
    $('.cve-sortable').removeClass('sort-asc sort-desc');
    $(this).addClass(`sort-${newDir}`);
    
    sortCVETable(column, newDir);
  });
  
  // Sources form handling
  $('#add-source-form').on('submit', function(e) {
    e.preventDefault();
    
    const newSource = {
      name: $('#source-name').val(),
      url: $('#source-url').val(),
      api_key: $('#source-api-key').val(),
      sync_frequency: parseInt($('#source-frequency').val()),
      is_active: $('#source-active').prop('checked')
    };
    
    // In a real implementation, this would be an AJAX POST
    console.log('Adding new source:', newSource);
    
    // For the prototype, just add to the table
    addSourceToTable(newSource);
    
    // Reset form
    $(this).trigger('reset');
    $('#add-source-section').hide();
  });
  
  // New source button
  $('#add-source-btn').on('click', function() {
    $('#add-source-section').toggle();
  });
  
  // Rule form handling
  $('#add-rule-form').on('submit', function(e) {
    e.preventDefault();
    
    const newRule = {
      name: $('#rule-name').val(),
      criteria: {
        severity: $('#rule-severity').val()
      },
      actions: {
        create_ticket: $('#create-ticket').prop('checked'),
        ticket_priority: $('#ticket-priority').val(),
        notify_admins: $('#notify-admins').prop('checked'),
        add_to_report: $('#add-to-report').prop('checked')
      },
      priority: parseInt($('#rule-priority').val()),
      is_active: $('#rule-active').prop('checked')
    };
    
    // In a real implementation, this would be an AJAX POST
    console.log('Adding new rule:', newRule);
    
    // For the prototype, just add to the table
    addRuleToTable(newRule);
    
    // Reset form
    $(this).trigger('reset');
    $('#add-rule-section').hide();
  });
  
  // New rule button
  $('#add-rule-btn').on('click', function() {
    $('#add-rule-section').toggle();
  });
  
  // Helper functions
  function filterCVEs(searchTerm, severity, status) {
    $('#cve-table tbody tr').each(function() {
      const cveId = $(this).find('td:nth-child(1)').text().toLowerCase();
      const description = $(this).find('td:nth-child(2)').text().toLowerCase();
      const rowSeverity = $(this).find('td:nth-child(3)').text();
      const rowStatus = $(this).find('td:nth-child(6)').text();
      
      const matchesSearch = !searchTerm || cveId.includes(searchTerm) || description.includes(searchTerm);
      const matchesSeverity = !severity || rowSeverity === severity;
      const matchesStatus = !status || rowStatus === status;
      
      if (matchesSearch && matchesSeverity && matchesStatus) {
        $(this).show();
      } else {
        $(this).hide();
      }
    });
  }
  
  function sortCVETable(column, direction) {
    const rows = $('#cve-table tbody tr').toArray();
    
    rows.sort(function(a, b) {
      let aValue = $(a).find(`td:nth-child(${getColumnIndex(column)})`).text();
      let bValue = $(b).find(`td:nth-child(${getColumnIndex(column)})`).text();
      
      // Handle numeric values
      if (column === 'cvss') {
        aValue = parseFloat(aValue);
        bValue = parseFloat(bValue);
      }
      
      // Handle dates
      if (column === 'published') {
        aValue = new Date(aValue);
        bValue = new Date(bValue);
      }
      
      if (direction === 'asc') {
        return aValue > bValue ? 1 : -1;
      } else {
        return aValue < bValue ? 1 : -1;
      }
    });
    
    // Reappend sorted rows
    $('#cve-table tbody').empty();
    $.each(rows, function(index, row) {
      $('#cve-table tbody').append(row);
    });
  }
  
  function getColumnIndex(columnName) {
    const indices = {
      'cve_id': 1,
      'severity': 3,
      'cvss': 4,
      'published': 5,
      'status': 6
    };
    return indices[columnName] || 1;
  }
  
  function addSourceToTable(source) {
    const now = new Date().toLocaleString();
    
    const newRow = `
      <tr>
        <td>${source.name}</td>
        <td>${source.url}</td>
        <td>Every ${source.sync_frequency} hours</td>
        <td>Never</td>
        <td><span class="badge badge-pending">PENDING</span></td>
        <td><span class="badge badge-${source.is_active ? 'success' : 'inactive'}">${source.is_active ? 'Active' : 'Inactive'}</span></td>
        <td class="text-right">
          <button class="btn btn-sm btn-primary sync-source-btn"><i class="fas fa-sync"></i></button>
          <button class="btn btn-sm btn-danger delete-source-btn"><i class="fas fa-trash"></i></button>
        </td>
      </tr>
    `;
    
    $('#sources-table tbody').append(newRow);
  }
  
  function addRuleToTable(rule) {
    const severityClass = {
      'CRITICAL': 'danger',
      'HIGH': 'warning',
      'MEDIUM': 'info',
      'LOW': 'primary'
    }[rule.criteria.severity] || 'primary';
    
    const actionLabels = [];
    if (rule.actions.create_ticket) actionLabels.push('<span class="badge badge-purple">Create Ticket</span>');
    if (rule.actions.notify_admins) actionLabels.push('<span class="badge badge-info">Notify</span>');
    if (rule.actions.add_to_report) actionLabels.push('<span class="badge badge-success">Report</span>');
    
    const newRow = `
      <tr>
        <td>${rule.priority}</td>
        <td>${rule.name}</td>
        <td><span class="badge badge-${severityClass}">${rule.criteria.severity}</span></td>
        <td>${actionLabels.join(' ')}</td>
        <td><span class="badge badge-${rule.is_active ? 'success' : 'inactive'}">${rule.is_active ? 'Active' : 'Inactive'}</span></td>
        <td class="text-right">
          <button class="btn btn-sm btn-primary edit-rule-btn">Edit</button>
          <button class="btn btn-sm btn-danger delete-rule-btn"><i class="fas fa-trash"></i></button>
        </td>
      </tr>
    `;
    
    $('#rules-table tbody').append(newRow);
  }
  
  // Load mock data for the prototype
  loadMockData();
  
  function loadMockData() {
    // This would be replaced by AJAX calls to the GLPI API in a real implementation
    const mockCVEs = [
      {
        id: 1,
        cve_id: 'CVE-2024-1234',
        description: 'A remote code execution vulnerability in Apache Log4j library affecting versions 2.0-beta9 to 2.14.1',
        cvss_score: 10.0,
        cvss_vector: 'CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:C/C:H/I:H/A:H',
        severity: 'CRITICAL',
        published_date: '2024-05-12',
        modified_date: '2024-05-14',
        status: 'NEW',
        references: [
          'https://nvd.nist.gov/vuln/detail/CVE-2024-1234',
          'https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2024-1234'
        ],
        affected_products: ['Apache Log4j 2.0-2.14.1', 'Applications using affected Log4j versions'],
        date_creation: '2024-05-15',
        date_mod: '2024-05-15'
      },
      {
        id: 2,
        cve_id: 'CVE-2024-5678',
        description: 'SQL Injection vulnerability in WordPress plugin Contact Form 7 versions prior to 5.7.2',
        cvss_score: 8.8,
        cvss_vector: 'CVSS:3.1/AV:N/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H',
        severity: 'HIGH',
        published_date: '2024-05-10',
        modified_date: '2024-05-11',
        status: 'ANALYZED',
        references: [
          'https://nvd.nist.gov/vuln/detail/CVE-2024-5678',
          'https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2024-5678'
        ],
        affected_products: ['WordPress Contact Form 7 < 5.7.2'],
        date_creation: '2024-05-12',
        date_mod: '2024-05-12'
      },
      {
        id: 3,
        cve_id: 'CVE-2024-9012',
        description: 'Privilege escalation vulnerability in Microsoft Windows 11 affecting kernel-mode drivers',
        cvss_score: 7.8,
        cvss_vector: 'CVSS:3.1/AV:L/AC:L/PR:L/UI:N/S:U/C:H/I:H/A:H',
        severity: 'HIGH',
        published_date: '2024-05-08',
        modified_date: '2024-05-09',
        status: 'ASSIGNED',
        references: [
          'https://nvd.nist.gov/vuln/detail/CVE-2024-9012',
          'https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2024-9012'
        ],
        affected_products: ['Microsoft Windows 11 21H2', 'Microsoft Windows 11 22H2'],
        date_creation: '2024-05-10',
        date_mod: '2024-05-13'
      },
      {
        id: 4,
        cve_id: 'CVE-2024-3456',
        description: 'Cross-site scripting (XSS) vulnerability in jQuery UI Dialog component prior to version 1.13.2',
        cvss_score: 5.4,
        cvss_vector: 'CVSS:3.1/AV:N/AC:L/PR:N/UI:R/S:U/C:L/I:L/A:N',
        severity: 'MEDIUM',
        published_date: '2024-05-05',
        modified_date: '2024-05-06',
        status: 'RESOLVED',
        references: [
          'https://nvd.nist.gov/vuln/detail/CVE-2024-3456',
          'https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2024-3456'
        ],
        affected_products: ['jQuery UI < 1.13.2'],
        date_creation: '2024-05-07',
        date_mod: '2024-05-15'
      },
      {
        id: 5,
        cve_id: 'CVE-2024-7890',
        description: 'Information disclosure vulnerability in OpenSSL 3.0.0 through 3.1.1',
        cvss_score: 3.7,
        cvss_vector: 'CVSS:3.1/AV:N/AC:H/PR:N/UI:N/S:U/C:L/I:N/A:N',
        severity: 'LOW',
        published_date: '2024-05-02',
        modified_date: '2024-05-03',
        status: 'ANALYZED',
        references: [
          'https://nvd.nist.gov/vuln/detail/CVE-2024-7890',
          'https://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-2024-7890'
        ],
        affected_products: ['OpenSSL 3.0.0-3.1.1'],
        date_creation: '2024-05-04',
        date_mod: '2024-05-10'
      }
    ];
    
    // Populate CVE table
    mockCVEs.forEach(cve => {
      const severityClass = {
        'CRITICAL': 'danger',
        'HIGH': 'warning',
        'MEDIUM': 'info',
        'LOW': 'primary'
      }[cve.severity];
      
      const statusClass = {
        'NEW': 'info',
        'ANALYZED': 'warning',
        'ASSIGNED': 'primary',
        'RESOLVED': 'success'
      }[cve.status];
      
      const row = `
        <tr>
          <td><a href="#" class="cve-link">${cve.cve_id}</a></td>
          <td>${cve.description.substring(0, 50)}...</td>
          <td><span class="badge badge-${severityClass}">${cve.severity}</span></td>
          <td>${cve.cvss_score.toFixed(1)}</td>
          <td>${cve.published_date}</td>
          <td><span class="badge badge-${statusClass}">${cve.status}</span></td>
          <td class="text-right">
            <button class="btn btn-sm btn-primary view-cve-btn">View</button>
            <button class="btn btn-sm btn-purple create-ticket-btn">Create Ticket</button>
            <a href="${cve.references[0]}" target="_blank" class="btn btn-sm btn-secondary"><i class="fas fa-external-link-alt"></i></a>
          </td>
        </tr>
      `;
      
      $('#cve-table tbody').append(row);
    });
    
    // Update dashboard counters
    const criticalCount = mockCVEs.filter(cve => cve.severity === 'CRITICAL').length;
    const highCount = mockCVEs.filter(cve => cve.severity === 'HIGH').length;
    const pendingCount = mockCVEs.filter(cve => cve.status === 'NEW').length;
    const resolvedCount = mockCVEs.filter(cve => cve.status === 'RESOLVED').length;
    
    $('#critical-count').text(criticalCount);
    $('#high-count').text(highCount);
    $('#pending-count').text(pendingCount);
    $('#resolved-count').text(resolvedCount);
    
    // Set progress bars
    $('#critical-progress').css('width', `${(criticalCount / mockCVEs.length) * 100}%`);
    $('#high-progress').css('width', `${(highCount / mockCVEs.length) * 100}%`);
    $('#pending-progress').css('width', `${(pendingCount / mockCVEs.length) * 100}%`);
    $('#resolved-progress').css('width', `${(resolvedCount / mockCVEs.length) * 100}%`);
    
    // Populate recent vulnerabilities table
    mockCVEs.slice(0, 5).forEach(cve => {
      const severityClass = {
        'CRITICAL': 'danger',
        'HIGH': 'warning',
        'MEDIUM': 'info',
        'LOW': 'primary'
      }[cve.severity];
      
      const statusClass = {
        'NEW': 'info',
        'ANALYZED': 'warning',
        'ASSIGNED': 'primary',
        'RESOLVED': 'success'
      }[cve.status];
      
      const row = `
        <tr>
          <td><a href="#" class="cve-link">${cve.cve_id}</a></td>
          <td><span class="badge badge-${severityClass}">${cve.severity}</span></td>
          <td>${cve.cvss_score.toFixed(1)}</td>
          <td>${cve.published_date}</td>
          <td><span class="badge badge-${statusClass}">${cve.status}</span></td>
        </tr>
      `;
      
      $('#recent-cve-table tbody').append(row);
    });
  }
});